<?php
/**
 * The Passwd_procopen class provides a procopen implementation of the
 * passwd system.
 *
 * Any script or program can be supplied as the 'program' attribute
 * value of the params argument.  The username, old password and new
 * password are written to the stdin of the process and then the
 * stdout and stderr of the process are read and combined with the
 * exit status value and returned to the caller if the status code is
 * not 0.
 *
 * $Horde: passwd/lib/Driver/procopen.php,v 1.2 2004/07/28 19:16:14 jan Exp $
 */
class Passwd_Driver_procopen extends Passwd_Driver {

    /**
     * Change the user's password by executing a user supplied
     * command.
     *
     * @param $user     User ID.
     * @param $oldpass  Old password.
     * @param $newpass  New password.
     *
     * @return boolean  True on success, false or error message on error.
     */
    function changePassword($user, $oldpass, $newpass)
    {
        global $conf;

        $descriptorspec = array(
            0 => array('pipe', 'r'),
            1 => array('pipe', 'w'),
            2 => array('pipe', 'e'));

        $output = '';

        $process = @proc_open($this->_params['program'], $descriptorspec, $pipes);
        if (is_resource($process)) {
            fwrite($pipes[0], "$user\n");
            fwrite($pipes[0], "$oldpass\n");
            fwrite($pipes[0], "$newpass\n");
            fclose($pipes[0]);
            while (!feof($pipes[1])) {
                $output .= fgets($pipes[1], 1024);
            }
            fclose($pipes[1]);
            while (!feof($pipes[2])) {
                $output .= fgets($pipes[2], 1024);
            }
            fclose($pipes[2]);
            $return_value = proc_close($process);
        } else {
            $output = '';
            $return_value = -1;
        }

        $output .= " (Exit Status: $return_value)";

        if ($return_value != 0) {
            return $output ? PEAR::raiseError($output) : false;
        } else {
            return true;
        }
    }

}
