<?php
/**
 * SAM Attributes File
 *
 * This file contains the attributes that SAM understands, and their
 * types. It may be safely edited by hand. Use attributes.php.dist as
 * a reference. Only a backend-supported subset of these values will
 * be presented to the user so it is safe to add more attributes.
 *
 * To disable an attribute (and prevent it from showing up to the user)
 * simply comment it out or remove it.
 *
 * General configuration is in 'conf.php'.
 * Backend storage is defined in 'backends.php'.
 * Default user preferences are defined in 'prefs.php'.
 *
 * $Horde: sam/config/attributes.php.dist,v 1.8 2004/06/28 02:07:11 chuck Exp $
 * The syntax of this array is as follows:
 *      label    - the text that the user will see attached to this field
 *      type     - one of the following:
 *                      - spacer            - header
 *                      - description       - html
 *                      - number            - int
 *                      - intlist           - text
 *                      - longtext          - countedtext
 *                      - address           - file
 *                      - boolean           - link
 *                      - email             - emailconfirm
 *                      - password          - passwordconfirm
 *                      - enum              - multienum
 *                      - radio             - set
 *                      - date              - time
 *                      - monthyear         - monthdayyear
 *                      - colorpicker       - sorter
 *                      - creditcard        - invalid
 *                      - stringlist
 *      required - boolean, true or false whether this field is mandatory
 *      readonly - boolean, true or false whether this editable
 *      desc     - any help text attached to the field
 *      params   - any other parameters that need to be passed to the field
 *                 such as a list to be used in the enum field, the row and
 *                 column sizing for a longtext field.
 */

$_attributes['tag_level'] = array(
    'label' => _("Tag Level"),
    'type' => 'number',
    'required' => false,
);
$_attributes['hit_level'] = array(
    'label' => _("Hit Level"),
    'type' => 'number',
    'required' => true,
);
// Alternatively levels may be defined as a
// list of values for simplifying user interface.
// For example:
//
//$_attributes['hit_level'] = array(
//    'label' => _("Required Hits"),
//    'type' => 'enum',
//    'required' => true,
//    'params' => array(array(
//        '' => _("Select Value:"),
//        '2' => _("2: Really Aggressive"),
//        '5' => _("5: Aggressive"),
//        '7' => _("7: Medium"),
//        '10' => _("10: Loose"),
//        '15' => _("15: Really Loose"),
//    )),
//);
$_attributes['kill_level'] = array(
    'label' => _("Kill Level"),
    'type' => 'number',
    'required' => false,
);
$_attributes['subject_tag'] = array(
    'label' => _("Subject Tag"),
    'type' => 'text',
    'required' => false,
);
$_attributes['rewrite_sub'] = array(
    'label' => _("Rewrite Subject"),
    'type' => 'boolean',
    'required' => false,
);
// As an alternative UI, all boolean types may be
// configured as radio buttons.
// For example:
//
//$_attributes['rewrite_sub'] = array(
//    'label' => _("Rewrite Subject"),
//    'type' => 'radio',
//    'required' => false,
//    'params' => array(array(
//        'Y' => 'Yes',
//        'N' => 'No',
//    ));
//);
$_attributes['spam_quarantine'] = array(
    'label' => _("Spam Quarantine Address"),
    'type' => 'text',
    'required' => false,
);
$_attributes['report_safe'] = array(
    'label' => _("Modify messages tagged as spam:"),
    'type' => 'enum',
    'required' => false,
    'params' => array(array(0 => _("0: Only add X-Spam-* headers"),
                            1 => _("1: Attach the original message to the report"),
                            2 => _("2: Attach the original message as plain text to the report")))
);
$_attributes['skip_rbl'] = array(
    'label' => _("Skip RBL Checks"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['skip_virus'] = array(
    'label' => _("Skip Virus Checks"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['skip_spam'] = array(
    'label' => _("Skip Spam Checks"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['skip_banned'] = array(
    'label' => _("Skip Banned File Checks"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['skip_header'] = array(
    'label' => _("Skip Bad Header Checks"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['allow_virus'] = array(
    'label' => _("Receive Viruses"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['allow_spam'] = array(
    'label' => _("Receive Spam"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['allow_banned'] = array(
    'label' => _("Receive Banned Files"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['allow_header'] = array(
    'label' => _("Receive Bad Headers"),
    'type' => 'boolean',
    'required' => false,
);
$_attributes['spam_extension'] = array(
    'label' => _("Spam Folder"),
    'type' => 'text',
    'required' => false,
);
$_attributes['virus_extension'] = array(
    'label' => _("Virus Folder"),
    'type' => 'text',
    'required' => false,
);
$_attributes['banned_extension'] = array(
    'label' => _("Banned Files Folder"),
    'type' => 'text',
    'required' => false,
);
// The above three settings may alternatively be configured
// as either a drop-down list of currently existing folders
// or a hard-coded string chosen by a yes/no radio button.
// For both of these examples to work, your server must
// support address extension (such as plus addressing).
//
// Existing folders example:
//
//if ($GLOBALS['registry']->hasMethod('mail/folderlist')) {
//    $mailboxes = $GLOBALS['registry']->call('mail/folderlist');
//}
//if ($mailboxes && !is_a($mailboxes, 'PEAR_Error')) {
//    $_attributes['spam_extension'] = array(
//        'label' => _("File spam Into Mailbox:"),
//        'type' => 'enum',
//        'required' => false,
//        'params' => array(array())
//    );
//
//    foreach ($mailboxes as $mbox) {
//        if ($mbox['val'] != 'INBOX') {
//            $_attributes['spam_quarantine']['params'][0][] = $mbox['label'];
//        }
//    }
//}
//
// Admin-restricted folder example:
//
//$_attributes['spam_extension'] = array(
//    'label' => _("File Into <b>spamfilter</b> Mailbox?"),
//    'type' => 'radio',
//    'required' => false,
//    'params' => array(array(
//        'spamfilter' => 'Yes',
//        '' => 'No'
//    ))
//);
