<?php

/** Backend-specific 'false' value. */
define('_SAM_OPTION_OFF', '0');

/** Backend-specific 'true' value. */
define('_SAM_OPTION_ON',  '1');

/**
 * Base class for all SpamAssassin drivers.
 *
 * $Horde: sam/lib/Driver/spamd.php,v 1.3 2005/01/03 14:35:26 jan Exp $
 *
 * Copyright 2003-2005 Chris Bowlby <excalibur@hub.org>
 * Copyright 2003-2005 Max Kalika <max@horde.org>
 * Copyright 2004-2005 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Chris Bowlby <excalibur@hub.org>
 * @author  Max Kalika <max@horde.org>
 * @author  Jan Schneider <jan@horde.org>
 * @version $Revision$
 * @since   SAM 0.1
 * @package Sam
 * @abstract
 */
class SAM_Driver_spamd extends SAM_Driver {

    /**
     * List of the capabilities supported by this driver.
     *
     * @var array $_capabilities
     */

  // SpamAssassin 2 setup:
  //    var $_capabilities = array('hit_level',
  //                             'report_safe',
  //                             'rewrite_sub',
  //                             'subject_tag',
  //                             'skip_rbl',
  //                             'whitelist_to',
  //                             'whitelist_from',
  //                             'blacklist_to',
  //                             'blacklist_from');

  // SpamAssassin 3 setup:
  var $_capabilities = array('hit_level',
			     'report_safe',
			     'subject_tag',
			     'skip_rbl',
			     'whitelist_to',
			     'whitelist_from',
			     'blacklist_to',
			     'blacklist_from');
  

    /**
     * List of Sam to SpamAssassin options mappings.
     *
     * @var array $_option_map
     */
    
    // SpamAsssassin 2 setup:
    //    var $_option_map = array('hit_level' => 'required_hits',
    //                             'rewrite_sub' => 'rewrite_subject',
    //                             'skip_rbl' => 'skip_rbl_checks');

    // SpamAssassin 3 setup:
    var $_option_map = array('hit_level' => 'required_score',
			     'subject_tag' => 'rewrite_header subject',
			     'skip_rbl' => 'skip_rbl_checks');

    /**
     * Converts a Sam attribute to an option that SpamAssassin will
     * use.
     *
     * @access protected
     *
     * @param string $attribute  The SAM attribute to convert.
     *
     * @return string  The converted SpamAssassin option or the original
     *                 attribute if no match is found.
     */
    function _mapAttributeToOption($attribute)
    {
        return isset($this->_option_map[$attribute])
               ? $this->_option_map[$attribute] : $attribute;
    }

    /**
     * Converts a SpamAssassin option to a Sam attribute.
     *
     * @access protected
     *
     * @param string $option  The SpamAssassin option to convert.
     *
     * @return string  The converted Sam attribute or the original option if
     *                 no match is found.
     */
    function _mapOptionToAttribute($option)
    {
        $attribute_map = array_flip($this->_option_map);
        return isset($attribute_map[$option])
               ? $attribute_map[$option] : $option;
    }

}
