<?php
/**
 * $Horde: turba/config/sources.php.dist,v 1.97 2004/11/17 20:36:41 chuck Exp $
 *
 * This file is where you specify the sources of contacts available to users at
 * your installation.  It contains a large number of EXAMPLES.  Please remove
 * or comment out those examples that YOU DON'T NEED.
 * There are a number of properties that you can set for each server,
 * including:
 *
 * title:    This is the common (user-visible) name that you want displayed in
 *           the contact source drop-down box.
 *
 * type:     The types 'ldap', 'sql', 'imsp' and 'prefs' are currently
 *           supported.  Preferences-based addressbooks are not intended for
 *           production installs unless you really know what you're doing -
 *           they are not searchable, and they won't scale well if a user has a
 *           large number of entries.
 *
 * params:   These are the connection parameters specific to the contact
 *           source.  See below for examples of how to set these.
 *
 * map:      This is a list of mappings from the standard Turba attribute names
 *           (on the left) to the attribute names by which they are known in
 *           this contact source (on the right).  Turba also supports composite
 *           fields.  A composite field is defined by mapping the field name to
 *           an array containing a list of component fields and a format string
 *           (similar to a printf() format string).  Here is an example:
 *           ...
 *           'name' => array('fields' => array('firstname', 'lastname'),
 *                           'format' => '%s %s'),
 *           'firstname' => 'object_firstname',
 *           'lastname' => 'object_lastname',
 *           ...
 *
 * tabs:     All fields can be grouped into tabs with this optional entry. This
 *           list is multidimensional hash, the keys are the tab titles.  Here
 *           is an example:
 *           'tabs' => array(
 *               'Names' => array('firstname', 'lastname', 'alias'),
 *               'Addresses' => array('homeAddress', 'workAddress')
 *           );
 *
 * search:   A list of Turba attribute names that can be searched for this
 *           source.
 *
 * strict:   A list of native field/attribute names that must always be matched
 *           exactly in a search.
 *
 * public:   If set to true, this source will be available to all users.  See
 *           also 'readonly' -- public=true readonly=false means writable by
 *           all users!
 *
 * readonly: If set to true, this source can only be modified by users on the
 *           'admin' list.
 *
 * admin:    A list (array) of users that are allowed to modify this source, if
 *           it's marked 'readonly'.
 *
 * export:   If set to true, this source will appear on the Export menu,
 *           allowing users to export the contacts to a CSV (etc.) file.
 *
 * charset:  Some LDAP servers use UTF8 for encoding. See below for examples.
 *
 * Here are some example configurations:
 */

// CLWELLS - 03/22/06 - MAKING CLUSTER NICE
require_once( 'DB.php' );
$iworx_ini = parse_ini_file( '/usr/local/interworx/lib/horde/iworx.ini', true );
$dsn = DB::parseDSN( $iworx_ini['horde']['dsn'] );

$dsn['table'] = 'turba_objects';
// CLWELLS - END
$params = array_merge($conf['sql'], array('table' => 'turba_objects'));
$cfgSources['localsql'] = array(
				'title' => 'My Addressbook',
				'type' => 'sql',
                                'browse' => '1',
				'params' => $params,
				'map' => array(
					       '__key' => 'object_id',
					       '__owner' => 'owner_id',
					       '__type' => 'object_type',
					       '__members' => 'object_members',
					       'name' => 'object_name',
					       'email' => 'object_email',
					       'homeAddress' => 'object_homeaddress',
					       'workAddress' => 'object_workaddress',
					       'homePhone' => 'object_homephone',
					       'workPhone' => 'object_workphone',
					       'cellPhone' => 'object_cellphone',
					       'fax' => 'object_fax',
					       'title' => 'object_title',
					       'company' => 'object_company',
					              'notes' => 'object_notes'
					       ),
				'search' => array(
						  'name',
						    'email'
						  ),
				'strict' => array(
						    'object_id'
						    ),
				'public' => false,
				'readonly' => false,
				'admin' => array(),
				'export' => true
				);

/**
 * A preferences-based adressbook. This will always be private. You
 * can add any attributes you like to the map and it will just work;
 * you can also create multiple prefs-based addressbooks by changing
 * the 'name' parameter. This is best for addressbooks that are
 * expected to remain small; it's not the most efficient, but it can't
 * be beat for getting up and running quickly, especially if you
 * already have Horde preferences working. Note that it is not
 * searchable, though - searches will simply return the whole
 * addressbook.
 */
$cfgSources['prefs'] = array(
			     'title' => _("Private Address Book"),
			     'type' => 'prefs',
			     'params' => array(
					       'name' => 'prefs',
					       'charset' => NLS::getCharset()
					       ),
			     'map' => array(
					    '__key' => 'id',
					    '__type' => '_type',
					    '__members' => '_members',
					    '__uid' => 'uid',
					    'name' => 'name',
					    'email' => 'mail',
					    'alias' => 'alias'
					    ),
			     'search' => array(
					       'name',
					       'email',
					       'alias'
					       ),
			     'strict' => array(
					       'id',
					       '_type',
					       ),
			     'public' => false,
			     'readonly' => false,
			     'export' => true
			     );

