<?php
/* $Horde: vacation/lib/Driver.php,v 1.2.2.4 2003/05/09 21:51:53 chuck Exp $
 *
 * Copyright 2001-2003 Eric Rostetter and Mike Cochrane
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 * Vacation_Driver:: defines an API for implementing vacation backends
 * for the vacation module.
 *
 * Vacation_Driver:: defines an API for implementing vacation backends
 * for the vacation module.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @author  Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @package Vacation
 */
require_once '/home/interworx/lib/pear/DB.php';
require_once '/home/interworx/include/IWorxDebug.php';
require_once '/home/interworx/include/IWorxDbUtil.php';

// TODO: Figure out a way of Horde connection to InterWorx without screwing
//       up horde's file inclusion.  It kinda sucks that we have to include
//       these InterWorx files here, it would be nice if this could all be done
//       via a command line script or something.

class Vacation_Driver_interworx extends Vacation_Driver {

    var $dsn;

    /**
     * Constructs a new ftp qmail Vacation_Driber object.
     *
     * @param array  $params    A hash containing connection parameters.
     */
    function Vacation_Driver_interworx($params = array())
      {
      	$iworx_ini = parse_ini_file( '/home/interworx/lib/horde/iworx.ini', true );
      	$this->dsn = $iworx_ini['vpopmail']['dsn'];
        $this->params = $params;
      }

    /**
     * Setup vacation notices for a user.
     *
     * @param string    $user       The username to enable vacation notices for.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password for the user.
     * @param string    $message    The text of the vacation notice.
     * @param string    $alias      The email alias to pass to vacation
     *
     * @return boolean  Returns true on success, false on error.
     */
    function setVacation($user, $realm = 'default', $pass = '', $message, $alias)
    {
      $domain = $realm;

      if( $realm == 'default' ) {
      	$regs = array();
      	ereg( "@(.*)", Auth::getAuth(), $regs );
      	$domain = $regs[1];
      }

      $domain = $this->_translatePointerDomain( $domain );

      if( !$this->_checkLogin( $user, $domain, $pass ) ) {
        $this->err_str = _("Check your username and password.");
      	return false;
      }

      $tmpfname = tempnam("", "VACA");
      $handle = fopen($tmpfname, "w");
      fwrite($handle, $message);
      fclose($handle);

      $cmd = "/home/interworx/lib/horde/setvacation --set-vacation-message=$tmpfname --user=$user --domain=$domain";

      exec( $cmd, $result, $retval );

      $db = IWorxDbUtil::db_connect( $this->dsn );

      $sql = "DELETE FROM valias WHERE alias = '$user' AND domain = '$domain' AND valias_line LIKE '|%vacation.sh%'";
      $db->query( $sql );

      $sql = "INSERT INTO valias (alias, domain, valias_line) VALUES ('$user', '$domain', '|/home/interworx/lib/autorespond/vacation.sh' )";
      $db->query( $sql );

      $sql = "SELECT count(*) from valias WHERE alias = '$user' AND domain = '$domain' AND valias_line LIKE '|%deliverquota.sh%'";
      $exists = $db->getOne( $sql );

      if( $exists == 0 ) {
      	$sql = "INSERT INTO valias (alias, domain, valias_line) VALUES ('$user', '$domain', '|/home/interworx/lib/autorespond/deliverquota.sh' )";
      	$db->query( $sql );
      }

      unlink( $tmpfname );

      return true;
    }

    /**
     * check the user/pass and make sure we're legit
     *
     * @param user
     * @param pass
     */

    function _checkLogin( $user, $domain, $pass ) {
      $db = IWorxDbUtil::db_connect( $this->dsn );

      $sql = "SELECT pw_passwd
              FROM vpopmail
              WHERE pw_name   = '$user' AND
                    pw_domain = '$domain'";

      $md5 = $db->getOne( $sql );

      $salt     = substr( $md5, 0, 12 );
      $checkMd5 = crypt( $pass, $salt );

      return $checkMd5 == $md5;
    }

    /**
     * Get the parent domain if this is a master domain.
     *
     * @param domain
     */
    function _translatePointerDomain( $domain ) {
      if( is_readable( '/var/qmail/users/assign' ) ) {
        // check for pointer domains
        $assign = file( '/var/qmail/users/assign' );
        foreach( $assign as $line ) {
          $parts = preg_split( '/:/', $line );
          if( $parts[0] === "+{$domain}-" ) {
            $domain = $parts[1];
          }
        }
      }
      return $domain;
    }

    /**
     * Disables vacation notices for a user.
     *
     * @param string    $user       The user to disable vacation notices for.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function unsetVacation($user, $realm = 'default', $pass = '')
    {
      $domain = $realm;

      if( $realm == 'default' ) {
      	$regs = array();
      	ereg( "@(.*)", Auth::getAuth(), $regs );
      	$domain = $regs[1];
      }

      $domain = $this->_translatePointerDomain( $domain );

      if( !$this->_checkLogin( $user, $domain, $pass ) ) {
      	$this->err_str = _("Check your username and password.");
      	return( false );
      }

      $db = IWorxDbUtil::db_connect( $this->dsn );
      $sql = "DELETE FROM valias WHERE alias = '$user' AND domain = '$domain' AND valias_line LIKE '|%vacation.sh%'";
      $db->query( $sql );

      $sql = "SELECT count(*)
        FROM valias
        WHERE alias = '$user' AND
        domain = '$domain' AND
        valias_line like '&%'";

      $exists = $db->getOne( $sql );
      if( $exists == 0 ) {
      	$sql = "DELETE FROM valias
                WHERE alias = '$user' AND
                domain = '$domain' AND
                valias_line LIKE '|%deliverquota.sh%'";
      	$db->query( $sql );
      }
      return true;
    }

    /**
     * Retrieves current state of vacation notices for a user.
     *
     * @param string    $user       The user to check vacation status of.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password of the user.
     *
     * @return mixed    Returns true if vacation  is enabled for the user
     *                  or false if vacation is currently disabled.
     */
    function isEnabled($user, $realm = 'default', $password)
    {
      $domain = $realm;

      if( $realm == 'default' ) {
      	$regs = array();
      	ereg( "@(.*)", Auth::getAuth(), $regs );
      	$domain = $regs[1];
      }

      $domain = $this->_translatePointerDomain( $domain );

      $db = IWorxDbUtil::db_connect( $this->dsn );
      $sql = "SELECT COUNT(*) FROM valias WHERE alias = '$user' AND domain = '$domain' AND valias_line LIKE '|%vacation.sh%'";
      $cnt = $db->getOne( $sql );
      if( $cnt > 0 ) {
      	return( 'Y' );
      } else {
	     return( 'N' );
      }
    }

    /**
     * Retrieves current vacation setup.
     *
     * @param string    $user       The user to fetch the setup from.
     * @param string    $realm      The realm of the user.
     * @param string    $password   The password of the user.
     *
     * @return string   Returns a string containing the current forwarding mail address or
     *                  false on error.
     */
    function currentMessage($user, $realm = 'default', $password)
    {
      $domain = $realm;

      if( $realm == 'default' ) {
      	$regs = array();
      	ereg( "@(.*)", Auth::getAuth(), $regs );
      	$domain = $regs[1];
      }

      $domain = $this->_translatePointerDomain( $domain );

      assert( $domain != "" );
      $cmd = "/home/vpopmail/bin/vuserinfo -d {$user}@{$domain}";
      exec( $cmd, $result, $retval );
      $userdir = $result[0];
      if( @is_file( "{$userdir}/vacation" ) ) {
      	return( @file_get_contents( "{$userdir}/vacation" ) );
      } else {
	      return( "I'm on vacation and will not be reading my mail for a while.\nYour mail will be dealt with when I return." );
      }
      return false;
    }

   /**
     * Parse an email address list and return it in a known standard form.
     * This will attempt to add the domain (realm) to unqualified addresses
     * if the realm is non-blank and not 'default'.
     *
     * @param   $user       The email address.
     * @param   $realm      The domain/realm to add if none is present.
     * @return  string      The email address(es) on success, false on error.
     */
    function _make_email_address($user, $realm)
    {
        $domain = ($realm != 'default') ? $realm : '';
        $email = '';

      	if( $realm == 'default' ) {
      	  $regs = array();
      	  ereg( "@(.*)", Auth::getAuth(), $regs );
      	  $domain = $regs[1];
      	}

        require_once 'Mail/RFC822.php';
        $parsed_email = Mail_RFC822::parseAddressList($user, $domain, FALSE);
        if (is_array($parsed_email) && count($parsed_email) > 0) {
            for ($i=0; $i < count($parsed_email); $i++) {
               $email .= !empty($email) ? ',' : '';
               if (is_object($parsed_email[$i])) {
                 $email .= $parsed_email[$i]->mailbox;
                 $email .= !empty($parsed_email[$i]->host)
                        ? '@' . $parsed_email[$i]->host
                        : '';
              } else {
                 $email .= $parsed_email[$i];
              }
            }
        } else {
            $this->err_str = _("Can't parse your email address");
            $email = false;
        }

        return $email;
    }

}
