<?php
/**
 * $Horde: vacation/lib/Driver/sql.php,v 1.21 2005/01/03 14:35:41 jan Exp $
 *
 * Copyright 2001-2005 Ilya Krel and Mike Cochrane
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Vacation_Driver_sql:: implements the Vacation_Driver API for SQL servers.
 *
 * @author  Ilya Krel <mail@krel.org.org>
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision$
 * @since   Vacation 2.1
 * @package Vacation
 */
class Vacation_Driver_sql extends Vacation_Driver {

    /** SQL connection object. */
    var $_db;

    /** Boolean which contains the state of the SQL connection. */
    var $_connected = false;

    /**
     * Check if the realm has a specific configuration. If not, try to
     * fall back on the default configuration. If still not a valid
     * configuration then exit with an error.
     *
     * @param string    $realm      The realm of the user, or "default" if none.
     *                              Note: passed by reference so we can change
     *                              its value.
     */
    function checkConfig(&$realm)
    {
        // If no realm passed in, or no table config for the realm
        // passed in, then we fall back to the default realm
        if (empty($realm) || empty($this->params[$realm]['table'])) {
            $realm = 'default';
        }

        // If still no table,user_col,pass_col,message,subject,vacation then we have a misconfigured module.
        if (empty($this->params[$realm]['table']) ||
            empty($this->params[$realm]['user_col']) ||
            empty($this->params[$realm]['pass_col']) ||
            empty($this->params[$realm]['message']) ||
            empty($this->params[$realm]['subject']) ||
            empty($this->params[$realm]['vacation']) ) {
            $this->err_str = _("The vacation application is not properly configured.");
            return false;
        }

        return true;
    }

    /**
     * Enable a vacation message for a user.
     *
     * @param string    $user      The username to enable vacation for.
     * @param string    $realm     The realm of the user.
     * @param string    $password  The password of the user.
     * @param string    $message   The message to install.
     * @param string    $alias     The email alias for vacation to use --
     *                             Not yet implemented.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function setVacation($user, $realm, $password, $message, $alias)
    {
        // Make sure the configuration file is correct
        if (!$this->checkConfig($realm)) {
            return false;
        }

        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($realm);

        /* Determine if $message contains Subject: and if it does
         * split it. */
        if (preg_match("/^Subject: ([^\n]+)\n(.+)$/s", $message, $m='')) {
            $mysubject = $m[1];
            $mymessage = $m[2];
        } else {
            $mysubject = '';
            $mymessage = $message;
        }

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Check if an entry already exists and create one otherwise
        $query = 'SELECT ' . $this->params[$realm]['vacation'] . ' AS vacation';
        $query .= ' FROM ' . $this->params[$realm]['table'];
        $query .= ' WHERE ' . $this->params[$realm]['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->params[$realm]['pass_col'] . ' = ' . $this->_db->quote(md5($password));
        $result = $this->_db->query($query);
        if (!is_a($result, 'PEAR_Error')) {
            $query = 'INSERT INTO ' . $this->params[$realm]['table'];
            $query .= ' ( ' . $this->params[$realm]['vacation'];
            $query .= ' , ' . $this->params[$realm]['user_col'];
            $query .= ' , ' . $this->params[$realm]['pass_col'] . ' ) ';
            $query .= ' VALUES ( ' . $this->_db->quote('n');
            $query .= ' , ' . $this->_db->quote($user);
            $query .= ' , ' . $this->_db->quote(md5($password)) . ' ) ';
            $result = $this->_db->query($query);
        }

        /* Build the SQL query. */
        $query = 'UPDATE ' . $this->params[$realm]['table'];
        $query .= ' SET ' . $this->params[$realm]['vacation'] . ' = ' . $this->_db->quote('y');
        $query .= ' , ' . $this->params[$realm]['message'] . ' = ' . $this->_db->quote($mymessage);
        $query .= ' , ' . $this->params[$realm]['subject'] . ' = ' . $this->_db->quote($mysubject);
        $query .= ' WHERE ' . $this->params[$realm]['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->params[$realm]['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            if ($result === DB_OK) {
                $this->_disconnect();
                return true;
            } else {
                $this->_disconnect();
                return false;
            }
        } else {
            $this->_disconnect();
            return false;
        }
    }

    /**
     * Disable the vacation message for a user.
     *
     * @param string    $user      The username of the user.
     * @param string    $realm     The realm of the user.
     * @param string    $pass      The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function unsetVacation($user, $realm, $password)
    {
        // Make sure the configuration file is correct
        if (!$this->checkConfig($realm)) {
            return false;
        }

        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($realm);

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        /* Build the SQL query. */
        $query = 'UPDATE ' . $this->params[$realm]['table'];
        $query .= ' SET ' . $this->params[$realm]['vacation'] . ' = ' . $this->_db->quote('n');
        $query .= ' WHERE ' . $this->params[$realm]['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->params[$realm]['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            if ($result === DB_OK) {
                $this->_disconnect();
                return true;
            } else {
                $this->_disconnect();
                return false;
            }
        } else {
            $this->_disconnect();
            return false;
        }
    }

    /**
     * Retrieve the current vacation details for the user.
     *
     * @param   $user        The username for which to retrieve details.
     * @param   $realm       The realm (domain) for the user.
     * @param   $password    The password for user.
     *
     * @return  mixed        Vacation details or false.
     */
    function _getUserDetails($user, $realm, $password)
    {
        // Make sure the configuration file is correct
        if (!$this->checkConfig($realm)) {
            return false;
        }

        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($realm);

        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        /* Build the SQL query. */
        $query = 'SELECT ' . $this->params[$realm]['vacation'] . ' AS vacation';
        $query .= ' , ' . $this->params[$realm]['message'] . ' AS message';
        $query .= ' , ' . $this->params[$realm]['subject'] . ' AS subject';
        $query .= ' FROM ' . $this->params[$realm]['table'];
        $query .= ' WHERE ' . $this->params[$realm]['user_col'] . ' = ' . $this->_db->quote($myuser);
        $query .= ' AND ' . $this->params[$realm]['pass_col'] . ' = ' . $this->_db->quote(md5($password));

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_array($row)) {
                $this->_disconnect();
                $row['message'] = 'Subject: ' . $row['subject'] . "\n" . $row['message'];
                return $row;
            } else {
                $this->_disconnect();
                return false;
            }
        } else {
            $this->_disconnect();
            return false;
        }
    }

    /**
     * Do an SQL connect and login as user with privilege to change
     * vacation.
     *
     * @return   boolean   True or False based on success of connect
     */
    function _connect($realm)
    {
        if (!$this->_connected) {
            // Build the params array to pass to DB
            $_args = array_merge($this->params, $this->params[$realm]);

            Horde::assertDriverConfig($_args, 'server',
                array('phptype', 'hostspec', 'username', 'database', 'table'),
                'vacation authentication SQL');

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($_args,
                                      array('persistent' => !empty($_args['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal(PEAR::raiseError(_("Unable to connect to SQL server.")), __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean true on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

    /**
     * Builds a username based on presense of realm
     * @return string user@realm or user
     */
    function _buildUsername($user, $realm)
    {
        if ($realm === 'default' ||
            $realm === '') {
            return $user;
        } else {
            return $user . "@" . $realm;
        }
    }

}
